/* 	CreateDocuments_BV21_NewGSGdata.js 
	Script for BrainVoyager QX 2.2
	By Rainer Goebel 1995-2009
	Modified for BVQX 2.1 (2009), for BrainVoyager 21 (07-2019) with new GSG data (08-2019).
*/

var ObjectsRawDataPath = BrainVoyager.BrowseDirectory("Please select the path to the BrainVoyager sample data");
var DTIDataPath = BrainVoyager.BrowseDirectory("Please select the path to the BrainVoyager DWI sample data");//"/Users/BVuser/Data/Human31dir/";

BrainVoyager.PrintToLog("Create FMR document...");
Create_FMR_Document();
BrainVoyager.PrintToLog("Create mosaic FMR document...");
Create_Mosaic_FMR_Document();
BrainVoyager.PrintToLog("Create VMR document...");
Create_VMR_Document();
BrainVoyager.PrintToLog("Create DMR document...");
Create_DMR_Document();
BrainVoyager.PrintToLog("Create AMR document...");
Create_AMR_Document();
 

// for mosaic EPI data
function Create_Mosaic_FMR_Document()
{
    BrainVoyager.PrintToLog(ObjectsRawDataPath + "/02-03BlockedRun1/dcm/" + "JudEck_20181128_BI_Exercises_sess4-0002-0001-00001.dcm");
    var docFMR = BrainVoyager.CreateDocumentMosaicFMR("DICOM", ObjectsRawDataPath + "/02-03BlockedRun1/dcm/" + "JudEck_20181128_BI_Exercises_sess4-0002-0001-00001.dcm", 
		291, 2, true, 64, "untitled-", false, 800, 800, 2, ObjectsRawDataPath, 1, 100, 100);
    docFMR.SaveAs( ObjectsRawDataPath + "/02-03BlockedRun1/" + "sub-01_ses-04_task-blocked_run- 1_bold.fmr" ); 
    docFMR.Close();
}

// for non-mosaic EPI data
function Create_FMR_Document()
{
    var firstfilename = BrainVoyager.BrowseFile("Please select the first file", "*.dcm");// "JudEck_20181128_BI_Exercises_sess4-0010-0001-00001.dcm";
 	var docFMR = BrainVoyager.CreateDocumentFMR( "DICOM", firstfilename, 1, 0, true, 36, "untitled", false, 64, 64, 2, ObjectsRawDataPath + "/10-11GrefieldMapping");
	docFMR.SaveAs("NonMosaicDICOM_SCRIPT.fmr");
	docFMR.Close();
}

// for anatomical data
function Create_VMR_Document()
{
    BrainVoyager.PrintToLog(ObjectsRawDataPath + "/08MPRAGE_1mm_Nondistorted/" + "JudEck_20181128_BI_Exercises_sess4-0008-0001-00001.dcm");
      var docVMR = BrainVoyager.CreateDocumentVMR( "DICOM", ObjectsRawDataPath + "/08MPRAGE_1mm_Nondistorted/" + "JudEck_20181128_BI_Exercises_sess4-0008-0001-00001.dcm", 192, false, 256, 256, 2 );
      docVMR.SaveAs( ObjectsRawDataPath + "/08MPRAGE_1mm_Nondistorted/" + "sub-01_ses-04_acq-nondistorted_T1w.vmr" );
    //docVMR.SaveAs( ObjectsRawDataPath + "/" + "sub-01_ses-04_acq-nondistorted_T1w.nii" ); // uncomment to save as NIfTI instead of VMR
	docVMR.CorrectIntensityInhomogeneities(); // IIHC can only be performed directly after VMR creation
	docVMR.Close();
}

// for DWI data
function Create_DMR_Document()
{
	var docDMR = BrainVoyager.CreateDocumentDMR("DICOM", DTIDataPath + "/" + "pimpul_070907_dti -0007-0001-00001.dcm", 31, 0, true, 75, "human31dir", false, 128, 128, 2, DTIDataPath);
	docDMR.SaveAs(DTIDataPath  + "/" + "HUMAN31DIR_SCRIPT.dmr");
	docDMR.TR = 8900;
	docDMR.InterSliceTime = 118;
	docDMR.TimeResolutionVerified = true;
	docDMR.Save();
	docDMR.Close();
}

// for localizers, etc.
function Create_AMR_Document()
{
    var docAMR = BrainVoyager.CreateDocumentAMR( "DICOM", ObjectsRawDataPath  + "/01Loc/" + "JudEck_20181128_BI_Exercises_sess4-0001-0001-00001.dcm", 9, false, 512, 512, 2 );
    docAMR.SaveAs(ObjectsRawDataPath  + "/" + "CG_SLICELOCALIZER_SCRIPT.amr");
    docAMR.Close();
}

/* Document properties:
TR (property)
	set via: docFMR.TR = 2000;
InterSliceTime (property)
NrOfVolumes (property)
PixelSizeOfSliceDimX (property)
PixelSizeOfSliceDimY (property)
InterSliceTime (property)
SliceThickness  (property)
GapThickness  (property)
VoxelResolutionVerified (property)
TimeResolutionVerified (property)
FileNameOfPreprocessdFMR (property)
	obtain via: var newname = docFMR.FileNameOfPreprocessdFMR;
*/
