/* GLM computations 
For BrainVoyager QX 2.1
 
03-04-19: changed 'BrainVoyagerQX' object into 'BrainVoyager' for BV21
 
*/

// choose from these constants
const FILETYPE_FMR = 1;
const FILETYPE_VTC = 2;
const NORMALISE_TIMECOURSE_PSC = 1;
const NORMALISE_TIMECOURSE_ZTRF = 2;
const NORMALIZE_TIMECOURSE_ZTRF_BASE = 3;
const GLMTYPE_SINGLESTUDY = 1;
const GLMTYPE_MULTISTUDY = 2;
const GLMTYPE_RANDOMEFFECTS = 3;
const PREDICTORS_SEPARATESTUDY = 1;
const PREDICTORS_SEPARATESUBJECTS = 2;

// select here the parameters for your GLM
var glmtype 				= GLMTYPE_MULTISTUDY;
var correctSerialCorr 		= true;
var normaliseTimeCourse 	= NORMALISE_TIMECOURSE_PSC; // choose from constants above
var predSeparation			= PREDICTORS_SEPARATESTUDY;
var filetype 				= FILETYPE_VTC;
var mainpath 				= BrainVoyager.BrowseDirectory("Please select the directory with the data") + "/";//"/Users/BVuser/Data/rfx/";

// for VTC-GLMs: write here the name of the anatomical file
var anatname = mainpath + "Average_Tal.vmr";//"anatrfx_TAL.vmr";
var mdmname = mainpath + "RFX_TAL.mdm";

// write here the design matrix names
var desmat1 = mainpath + "run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.sdm";
var desmat2 = mainpath + "run2_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.sdm";

// write here the filenames
var nroffiles = 10;
var filenames = new Array(nroffiles);
/*filenames[0] = mainpath + "S1_loc1_0002-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[1] = mainpath + "S1_loc2_0003-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[2] = mainpath + "S2_loc1_0005-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[3] = mainpath + "S2_loc2_0003-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[4] = mainpath + "S3_loc1_0003-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[5] = mainpath + "S3_loc2_0005-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[6] = mainpath + "S4_loc1_0003-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[7] = mainpath + "S4_loc2_0004-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[8] = mainpath + "S5_loc1_0002-0001-0001_SCCAI2.fmr_TAL.vtc";
filenames[9] = mainpath + "S5_loc2_0003-0001-0001_SCCAI2.fmr_TAL.vtc";*/
filenames[0] = mainpath + "Sub1_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[1] = mainpath + "Sub1_run2_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[2] = mainpath + "Sub2_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[3] = mainpath + "Sub2_run2_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[4] = mainpath + "Sub3_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[5] = mainpath + "Sub3_run2_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[6] = mainpath + "Sub4_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[7] = mainpath + "Sub4_run2_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[8] = mainpath + "Sub5_run1_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";
filenames[9] = mainpath + "Sub5_run2_SCSAI2_3DMCTS_LTR_THPFFT3c_TAL.vtc";


// please write below the design matrix file that corresponds to each file
var desmatfiles = new Array(nroffiles);
desmatfiles[0] = desmat1;
desmatfiles[1] = desmat2;
desmatfiles[2] = desmat1;
desmatfiles[3] = desmat2;
desmatfiles[4] = desmat1;
desmatfiles[5] = desmat2;
desmatfiles[6] = desmat1;
desmatfiles[7] = desmat2;
desmatfiles[8] = desmat1;
desmatfiles[9] = desmat2;

Generate_MDM_file(mdmname); // In BrainVoyager QX 2.1 only for VTCs
Compute_GLMs();
RunRandomEffectsGLM();

// ~~ don't change from here ~~

function Compute_GLMs() {
    
	var doc, funcfile, desmatfile;
	if (glmtype == GLMTYPE_SINGLESTUDY) {
		BrainVoyager.PrintToLog("Start single-study GLMs...");
		for (counter = 0; counter < nroffiles; counter++) {	 
	    		funcfile = filenames[counter];
	    		desmatfile = desmatfiles[counter];
			if (filetype == FILETYPE_FMR) {		
		    		BrainVoyager.PrintToLog("Opening " + funcfile + "..."); // functional data (*.fmr + *.stc)
		    		doc = BrainVoyager.OpenDocument(funcfile); 
			} else if (filetype == FILETYPE_VTC){
    		    		BrainVoyager.PrintToLog("Opening " + anatname + "..."); // anatomical data (*.vmr)
		    		doc = BrainVoyager.OpenDocument(anatname); 
    		    		BrainVoyager.PrintToLog("Linking " + funcfile + "..."); // functional data (*.vtc)
		    		doc.LinkVTC(funcfile); 
			}
			doc.CorrectForSerialCorrelations 		= correctSerialCorr;
			BrainVoyager.PrintToLog("Loading " + desmatfile + "...");
			BrainVoyager.PrintToLog("Correct serial correlations: " + (doc.CorrectForSerialCorrelations ? "yes" : "no"));
			doc.LoadSingleStudyGLMDesignMatrix(desmatfile);	// design matrix (*.rtc/*.sdm)
			doc.ComputeSingleStudyGLM();
			
			var glmname = removeextension(funcfile) + ".glm";		// bool 	rename(const QString &newName)
			BrainVoyager.PrintToLog("Saving " + glmname + "...");	
			doc.SaveGLM(glmname); 
			doc.ShowGLM(); 
		} // end counter		  
	
	} else if  (glmtype == GLMTYPE_MULTISTUDY) {
		
		 BrainVoyager.PrintToLog("Opening " + anatname + "..."); // anatomical data (*.vmr)
		 doc = BrainVoyager.OpenDocument(anatname); 
		 doc.LoadMultiStudyGLMDefinitionFile(mdmname);
		 doc.PSCTransformStudies 			= (normaliseTimeCourse == NORMALISE_TIMECOURSE_PSC);
		 doc.ZTransformStudies 				= (normaliseTimeCourse == NORMALISE_TIMECOURSE_ZTRF);
		 doc.ZTransformStudiesBaselineOnly 	= (normaliseTimeCourse == NORMALIZE_TIMECOURSE_ZTRF_BASE);
		 doc.SeparationOfSubjectPredictors 	= (predSeparation == PREDICTORS_SEPARATESUBJECTS);
		 doc.SeparationOfStudyPredictors 		= (predSeparation == PREDICTORS_SEPARATESTUDY);
		 BrainVoyager.PrintToLog("Predictor separation: " + (doc.SeparationOfSubjectPredictors ? "per subject" : (doc.SeparationOfStudyPredictors ? "per run":"not set")));
		 BrainVoyager.PrintToLog("Data normalization: " + (doc.ZTransformStudies ? "z-transform" : (doc.ZTransformStudiesBaselineOnly ? "z-transform (baseline)" : 
										(doc.PSCTransformStudies ? "percent signal change " : "not set"))));
		 doc.ComputeMultiStudyGLM();
		 var glmname = removeextension(mdmname) + ".glm";		
		 BrainVoyager.PrintToLog("Saving " + glmname + "...");	
		 doc.SaveGLM(glmname);
		 doc.ShowGLM();
	}
}

function Generate_MDM_file(mdmname) {

    BrainVoyager.ShowLogTab();
	BrainVoyager.PrintToLog("Start creating MDM...\nOpening anatomical file: " + anatname);
    var anat = BrainVoyager.OpenDocument(anatname);
	if (anat == undefined) {
		BrainVoyager.PrintToLog("Invalid anatomical filename.");
	} else {
        anat.ClearMultiStudyGLMDefinition();
        for (counter = 0; counter < nroffiles; counter++) {
			BrainVoyager.PrintToLog("Adding:\n" + filenames[counter] + "\n" + desmatfiles[counter]);
			anat.AddStudyAndDesignMatrix(filenames[counter], desmatfiles[counter]);
        }
	//	var mdmname = removeextension(anatname) + ".mdm";
		BrainVoyager.PrintToLog("Name for MDM file: " + mdmname);
		anat.SaveMultiStudyGLMDefinitionFile(mdmname);
    }
}
	
function RunRandomEffectsGLM() {
    
    BrainVoyager.PrintToLog("Start random effects GLM... ");
    var anat = BrainVoyager.OpenDocument(anatname);
	anat.ClearMultiStudyGLMDefinition();
    anat.LoadMultiStudyGLMDefinitionFile(mdmname);
 	anat.PSCTransformStudies 			= (normaliseTimeCourse == NORMALISE_TIMECOURSE_PSC);
	anat.ZTransformStudies 				= (normaliseTimeCourse == NORMALISE_TIMECOURSE_ZTRF);
	anat.ZTransformStudiesBaselineOnly 	= (normaliseTimeCourse == NORMALIZE_TIMECOURSE_ZTRF_BASE);
	anat.SeparationOfSubjectPredictors 	= (predSeparation == PREDICTORS_SEPARATESUBJECTS);
	anat.SeparationOfStudyPredictors 	= (predSeparation == PREDICTORS_SEPARATESTUDY);
	BrainVoyager.PrintToLog("Predictor separation: " + (anat.SeparationOfSubjectPredictors ? "per subject" : (anat.SeparationOfStudyPredictors ? "per run":"not set")));
	BrainVoyager.PrintToLog("Data normalization: " + (anat.ZTransformStudies ? "z-transform" : (anat.ZTransformStudiesBaselineOnly ? "z-transform (baseline)" : 
										(anat.PSCTransformStudies ? "percent signal change " : "not set"))));
    anat.ComputeRFXGLM();
    var rfxglmname = removeextension(mdmname) + "_rfx.glm";
	BrainVoyager.PrintToLog("Saving " + rfxglmname + "...");	
    anat.SaveGLM(rfxglmname);
 	anat.ShowGLM();
}

function removeextension(filename) {
	var extlength = getextensionlength(filename);
	var name = "";
	for (i=0; i<(filename.length-extlength); i++) {
		letter = filename[i];
		name += letter;
	}	
	return name;
}

function getextensionlength(filename) {
	var extensionlength, letter, dotpos;
	for (i=filename.length; i>0; i--) {
		letter = filename[i];
		if (letter == ".") {
			dotpos = i;					
			break;
		}	
	}
	extensionlength = filename.length - dotpos;
	return extensionlength;
}