
/* 	Statistics.js
	Script for BrainVoyager QX 2.1, adapted for BrainVoyager 21 (11-04-19)
*/
/* Declarations */
var ObjectsRawDataPath = BrainVoyager.BrowseDirectory("Please select the directory of the sample experiment") + "/"; //"/Users/BVuser/Data/ObjectsDicomGSG/";
var RFXDataPath = BrainVoyager.BrowseDirectory("Please select the directory of the sample RFX experiment") + "/"; // "/Users/BVuser/Data/Exercise_RFX_ANOVA/";

/* This code is executed when clicking the 'Run' button */
try {
	
	RunSingleSubjectGLM(); // invoke function below 
	OverlayContrasts();
	RunMultiSubjectGLM(); 
	RunRandomEffectsGLM(); 

} catch (error) {
	BrainVoyager.PrintToLog("Error: " + error);
}

/* These functions can be invoked */
function RunSingleSubjectGLM() 
{
    try {
	var vmrname = BrainVoyager.BrowseFile("Please select the anatomical file", "*.vmr");
        var doc = BrainVoyager.OpenDocument(vmrname); // ObjectsRawDataPath + "CG_3DT1MPR_SCRIPT_TAL.vmr");
	var vtcname = BrainVoyager.BrowseFile("Please select the functional file", "*.vtc");
        doc.LinkVTC(vtcname); // ObjectsRawDataPath + "CG_OBJECTS_SCRIPT_TAL.vtc");
        doc.ClearDesignMatrix();
	var sdmname = BrainVoyager.BrowseFile("Please select the design matrix file", "*.sdm");
        doc.LoadSingleStudyGLMDesignMatrix(sdmname); // ObjectsRawDataPath + "CG_OBJECTS_FROMSCRIPT.sdm");
	var prtname = BrainVoyager.BrowseFile("Please select the stimulation protocol file", "*.prt");
        doc.LinkStimulationProtocol(prtname); // ObjectsRawDataPath + "CG_OBJECTS_FROMSCRIPT.prt");
        doc.CorrectForSerialCorrelations = 2; // If set to ”1”, AR(1) is used, if set to“2”, AR(2) is used.
        doc.ComputeSingleStudyGLM();
        doc.ShowGLM();
        doc.SaveGLM(ObjectsRawDataPath + "CG_OBJECTS_FROMSCRIPT.glm");
	} catch (error) {
		BrainVoyager.PrintToLog(error);
	}
}

function RunMultiSubjectGLM() 
{
	var vmrname = BrainVoyager.BrowseFile("Please select the anatomical file", "*.vmr");
        var doc = BrainVoyager.OpenDocument(vmrname); // RFXDataPath + "Average_Tal.vmr");
	doc.CorrectForSerialCorrelations = 2; // If set to ”1”, AR(1) is used, if set to“2”, AR(2) is used.
	doc.SeparationOfSubjectPredictors = true;
	doc.ZTransformStudies = true;
	var mdmname = BrainVoyager.BrowseFile("Please select the multi subject design matrix file", "*.mdm");
	doc.LoadMultiStudyGLMDefinitionFile(mdmname); // RFXDataPath + "MultiStudy_FROMSCRIPT.mdm");
	doc.ComputeMultiStudyGLM();
	doc.ShowGLM();
	doc.SaveGLM(RFXDataPath + "MultiStudy_FROMSCRIPT.glm");
}

function RunRandomEffectsGLM() 
{
	var vmrname = BrainVoyager.BrowseFile("Please select the anatomical file", "*.vmr");
       var doc = BrainVoyager.OpenDocument(vmrname); // RFXDataPath + "Average_Tal.vmr");
	var mdmname = BrainVoyager.BrowseFile("Please select the multi subject design matrix file", "*.mdm");
	doc.LoadMultiStudyGLMDefinitionFile(mdmname); // RFXDataPath + "MultiStudy_FROMSCRIPT.mdm");
	doc.PSCTransformStudies = true;
	doc.ComputeRFXGLM();
	doc.SaveGLM(RFXDataPath + "RFXGLM_FROMSCRIPT.glm");
}

function OverlayContrasts()
{
	var vmrname = BrainVoyager.BrowseFile("Please select the anatomical file", "*.vmr");
       var doc = BrainVoyager.OpenDocument(vmrname); // ObjectsRawDataPath + "CG_3DT1MPR_SCRIPT_CLEAN_TAL.vmr");
	doc.LoadGLM("CG_OBJECTS_FROMSCRIPT.glm");
	doc.ClearContrasts();
	doc.AddContrast("LVF > RVF");
	doc.SetContrastValue("LVF", +1);
	doc.SetContrastValue("RVF", -1);
//	alternatively: doc.SetContrastString("1 -1 0 0");
	// or ("Index" starts with 1)
//	doc.SetContrastValueAtIndex(1, +1);
//  doc.SetContrastValueAtIndex(2, -1);

	doc.ShowGLM();
}
